/*
 * This is the building block of ALL objects 
 * If you need to create a new character then it should be a child of this class
 * 
 * All ENEMIES, possible all objects later, functions will be called from this class
 * Since the functions are virtual, your children functions will override any functions here
 * You do not need to declare functions that will be local to the class; however,
 * If you need to have access to it from the parent there will have to be a virtual funtion with a NULL block listed here
*/

#ifndef _GAMEPIECE_H_
#define _GAMEPIECE_H_


class GamePiece
{
// Anything protected can be acessed by children, but not outside of the classes
// If it were private, the children couldn't access these variables either
protected:
	
/************************/
/*      VARIABLES       */
/***********************/

	// Do not try to initialize these local variables here
	// Must be done inside the initialize function
	coord _pos;
	VECTOR _speed;

	int _maxAnimation;
	imgContainer *_animation;
	int *_numFrames;

	int _curAnimation;
	int _curFrame;

public:

/************************/
/*     CONSTRUCTORS     */
/***********************/
	GamePiece()
	{
		this->initialize();
		this->loadAnimation();
	}
	GamePiece(coord xy)
	{
		this->initialize();
		this->loadAnimation();
		this->_pos=xy;
	}
/* these will come in later to recycle graphics to save processing and memory load
	GamePiece(coord xy, imgContainer *img)
	{
		initialize();
		loadAnimation(img, 1);
		this->_pos=xy;
	}

	GamePiece(imgContainer *img)
	{
		initialize();
		loadAnimation(img, 1);
	}
*/




/************************/
/*    DECONSTRUCTORS    */
/***********************/
	~GamePiece()
	{
		// I'm lazy
		// This will ensure all memory gets released when object is destoryed
	}




/************************/
/*       MUTATORS       */
/***********************/

	// This function will load an image from file into the corresponding slot for the animation type and frame position
	// The colorkey is used to specify the alpha channel; 0 is black
	// This will need to be modified if/when we change the image types
	virtual bool loadCell(int state=-1, int index=-1, string fName="", Uint32 colorkey=0)
	{
		//We are trying to enter into an array, valid indexes must exist
		if(state<0 || index<0 || fName.empty()) return false;
		
		// Load the image into the proper location
		this->_animation[state].sprite[index] = SDL_LoadBMP(fName.c_str());

		// if color key is 0 then use deafualt alpha channel; which is a pink color
		// otherwise, use what ever color key was suppllied by user
		if(!colorkey) colorkey=SDL_MapRGB( this->_animation[0].sprite[0]->format, 244, 8, 241);

		// Applys the alpha channel to the appropriate image
		// This means each image could have its own, unique alpha channel
		SDL_SetColorKey(this->_animation[state].sprite[index],SDL_SRCCOLORKEY, colorkey);

		// Changes the format of the image to recognize that we're using the alpha channel now
		this->_animation[state].sprite[index] = SDL_DisplayFormatAlpha(this->_animation[state].sprite[index]);

		// success! But probably should do a little more handeling
		return true;
	}
	
	// This is a general loader for the object
	// For the most part, this is hand crafted because the file names are arbitrary
	// If the images were indexed or referenced in another array this section could be more automated
	virtual bool loadAnimation()
	{
		// This is the number of different types of animation you like
		// ex: JUMP, RUN, WALK, ATTACK, etc..
		// In this example we only have one type; a sorta inbetween walk
		this->_maxAnimation=1;
		// For each type of animation there are N number of frames
		// This sets up the array that contains each animation's frame count
		this->_numFrames=new int[this->_maxAnimation];

		// For each type of animation you must specify how many frames are in it
		// A better example would be this:
		// this->numFrames[JUMP]=3;
		// this->numFrames[WALK]=5;
		// this->numFrames[RUN]=8;
		this->_numFrames[0]=1;

		// Create the memory space that will hold each animation type
		this->_animation=new imgContainer[this->_maxAnimation];

		// For each ANIMATION type create the space for the frames
		for(int _state=0; _state<this->_maxAnimation; _state++)
		{
			this->_animation[_state].sprite==new SDL_Surface*[this->_numFrames[_state]];

			// This will be used to aid in deconstruction
			// Not 100% correct, but the idea is to be able to reuse images and animations
			// If its a copy we want destroy the object and not the image its refering too
			// This will be worked on later .. don't worry about it
			this->_animation[_state].isCopy=false;
		}
		
		// This is hand coded but could be done automatically if the file names are indexed as well
		// This could be done in the directory or loaded into an array elsewhere
		this->loadCell(0,0,"images/walk_mid.bmp");

		return true;
	}
/* This was part of an idea to reuse animation .. it is not compatable in the newest representation of animation
	virtual bool loadAnimation(imgContainer img, int mxAnime)
	{
		img->isCopy=true;
		this->copy=true;
		this->_animation=new imgContainer* [this->_maxAnimation];
		for(int i=0; i<1; i++)
		{
			this->_animation[0]=new SDL_Surface[this->numFrames];
		}

		for(int r=0; r<rc.h; r++)
			for(int c=0; c<rc.w; c++)
				this->_sprite=surf;
		this->_dim=dimension(this->_sprite[0][0]->w, this->_sprite[0][0]->h);
	}
*/

	// All initalizations should take place here
	// If its listed as a variable, it needs to be handeled here
	// This will get called with every construction
	virtual void initialize()
	{
		this->_pos=coord(0,0);
		this->_speed=VECTOR(0,0);
		this->_maxAnimation=0;
		this->_animation=NULL;
		this->_numFrames=NULL;
		this->_curAnimation=0;
		this->_curFrame=0;
	}

	// This is the main funtion of the class
	// Everything gets updated each frame
	// This function makes since of what it means to "update" the object
	virtual void update()
	{
		this->processKeys();					// This object is modified by the keyboard, check how
		this->_pos=this->_pos+this->_speed;     // We have a velocity VECTOR that can be added to a position
		this->display();
	}

	// Each object will determine if key strokes will affect the object
	// If you don't have a use for keystorkes simple replace this block with '{return;}' in the CHILD CLASS
	virtual void processKeys()
	{
		// This is legacy form Sinzinger
		// Should be split only have KEYSTROKES and not ANY EVENT
		switch (event.type) 
		{
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) 
				{
					// Check for arrow keys and adjust
					case SDLK_LEFT:
						this->_speed.x=-this->_speed.speed;
						break;
					case SDLK_RIGHT:
						this->_speed.x=this->_speed.speed;
						break;
					case SDLK_UP:
						this->_speed.y=-this->_speed.speed;
      					break;
      				case SDLK_DOWN:
      					this->_speed.y=this->_speed.speed;
      					break;
						/*
      				case SDLK_q:
      					state = GAME_STATE_TITLE;
      					break;
      				case SDLK_e:
      					state = GAME_STATE_QUIT;
      					break;
    				case SDLK_w:
      					state = GAME_STATE_WIN;
      					break;
    				case SDLK_l:
      					state = GAME_STATE_LOSE;
      					break;
						*/
      				default:
      					break;
    			}
				break;
			
			case SDL_KEYUP:
				switch( event.key.keysym.sym )
				{
					// Stops movement when the correct key is lifted
					// Noticed some bugs from this when holding down to many keys
					// Simple logic fix for later
    				case SDLK_LEFT:
      					this->_speed.x=0;
      					break;
    				case SDLK_RIGHT:
      					this->_speed.x=0;
      					break;
    				case SDLK_UP:
      					this->_speed.y=0;
      					break;
    				case SDLK_DOWN:
      					this->_speed.y=0;
      					break;
    				default:
      					break;
    			}
				break;
			case SDL_QUIT:
    			state = GAME_STATE_QUIT;
		}
	}


/************************/
/*       ACESSORS       */
/***********************/

	// This is how and where a frame from the animation will be displayed
	// Monsters can be reused with different graphics by simply creating a new class with only a new disply function
	virtual void display()
	{
		SDL_Rect block;
		block.x = this->_pos.x;
		block.y = this->_pos.y;

		// Here '_curAnimation' and '_curFrame' are static, but the animator will increment these when necessary
		SDL_BlitSurface(this->_animation[this->_curAnimation].sprite[this->_curFrame], NULL, screen, &block);
	}

	// Each child should create their own collision detection
	// Since the gamepiece is basic we don't really care about collisions; therefore, return no collision
	virtual bool collision()
	{
		// Modify as children
		return false;
	}

};

#endif